<?php

namespace Drupal\yaml2php\Form;

use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\Core\Render\Markup;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html as HtmlUtility;
use Drupal\Component\Utility\Xss;
use Drupal\Component\Serialization\Yaml;

/**
 * Class ConverterForm.
 */
class ConverterForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'converter_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $through_interface = Link::fromTextAndUrl($this->t('through the interface'), Url::fromRoute('config.export_single'))->toString();
    $form['configuration_object_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Configuration object name'),
      '#description' => $this->t('For example &#039;filter.format.full_html&#039;. Accepted both with or without a &#039;.yml&#039; file extension.'),
      '#maxlength' => 128,
      '#size' => 64,
    ];
    $form['parent_keys'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Parent keys'),
      '#description' => $this->t('Parent keys from highest to one above the snippet. One per line. Example: &#039;filters&#039;.'),
      '#rows' => 8,
    ];
    $form['yaml_snippet'] = [
      '#type' => 'textarea',
      '#title' => $this->t('YAML snippet'),
      '#description' => $this->t('The snippet can be only a part of a YAML, either from a file or from the database @through_interface. There is no need to remove preceding whitespace.<br /><strong>Leave empty to capture the whole configuration object.</strong>', ['@through_interface' => $through_interface]),
      '#rows' => 18,
    ];
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Convert to PHP'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    // Validatre the object name but only if given.
    if (trim($form_state->getValue('configuration_object_name')) && !preg_match('/^[\w\-. ]+$/', $form_state->getValue('configuration_object_name'))) {
      $form_state->setErrorByName('configuration_object_name', $this->t('Invalid object name.'));
    }

  }

  /**
   * Display converted input at top of form page.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $input = $form_state->getValues();
    $input_config_object = preg_replace('/\.yml$/', '', HtmlUtility::escape($input['configuration_object_name']));
    // Initialize variable.
    $output = NULL;
    // Rules to sanitize output according to the Drupal coding standard later.
    $target = [
      '/array \(/',
      '/\)/',
      '/true/',
      '/false/',
      '/=>[ \t]+' . PHP_EOL . '[ \t]+\[' . PHP_EOL . '[ \t]+\],/',
      '/=>[ \t]+' . PHP_EOL . '/',
    ];
    $replacement = [
      '[',
      ']',
      'TRUE',
      'FALSE',
      '=> [],',
      '=>' . PHP_EOL,
    ];
    // Generate a page intro with data overview.
    $html = '<h3>' . $this->t('Configuration object name') . '</h3><p>' . $input_config_object . '</p>';
    if (trim($input['yaml_snippet'])) {
      $parent_keys = array_map('trim', array_values(array_filter(explode(PHP_EOL, HtmlUtility::escape($input['parent_keys'])))));
      $keys = NULL;
      foreach ($parent_keys as $parent_key) {
        $keys .= '<li>' . $parent_key . '</li>' . PHP_EOL;
      }
      $html .= '<h3>' . $this->t('Parent keys') . '</h3><p><ul>' . $keys . '</ul></p>';

      // Parse the YAML snippet and convert it to a PHP array.
      $yamlDecoded = Yaml::decode($this->unindent(Xss::filterAdmin($input['yaml_snippet'])));
      // Create the name of the configuration object to set.
      $parent_keys_string = implode('.', $parent_keys);
      // Generate the PHP code snippet.
      $html .= $this->t('<h2>PHP code to copy and paste</h2>Based on the configuration as set in the database. Remember to export the configuration to the coresponding YAML file and commit.');
      foreach ($yamlDecoded as $key => $value) {
        $config_object = print_r(var_export($value, TRUE), TRUE);
        $output .= "$$key = " . preg_replace($target, $replacement, $config_object) . ";
\Drupal::configFactory()->getEditable('$input_config_object')
  ->set('$parent_keys_string.$key', $$key)
  ->save();" . PHP_EOL . PHP_EOL;
      }
      $html .= '<pre>' . highlight_string("<?php
/**
 * @file
 * Set part of a configuration object in the database.
 *
 * Generated with https://drupal.org/project/yaml2php.
 */
" . $output, TRUE) . '</pre><h2>Submit another</h2>';
    }
    else {
      // Parse the YAML file and convert it to a PHP array.
      $yamlDecoded = Yaml::decode(file_get_contents(dirname(DRUPAL_ROOT) . '/config/' . CONFIG_SYNC_DIRECTORY . '/' . $input_config_object . '.yml'));
      // Remove the 'UUID' and '_core'. Not needed in the generated code.
      unset($yamlDecoded['uuid']);
      unset($yamlDecoded['_core']);
      // Create a variable name based on the configuration object name.
      $dynamic_var = '$' . str_replace('.', '_', $input_config_object);
      $config_object = print_r(var_export($yamlDecoded, TRUE), TRUE);
      $output .= preg_replace($target, $replacement, $config_object);
      $html .= $this->t('<h2>PHP code to copy and paste</h2>Based on the configuration YAML file.');
      $html .= '<pre>' . highlight_string("<?php
/**
 * @file
 * Set a complete configuration object in the database.
 *
 * Generated with https://drupal.org/project/yaml2php.
 */
$dynamic_var = $output;
\Drupal::configFactory()->getEditable('$input_config_object')
  ->setData($dynamic_var)
  ->save();", TRUE) . '</pre><h2>Submit another</h2>';
    }
    $html = Markup::create($html);
    $this->messenger()->addMessage($html, 'custom');
  }

  /**
   * Remove excessive indenting.
   *
   * Remove the first space of each line in a loop until any line does not
   * start with a space anymore. Needed to get a valid YAML snippet.
   *
   * @param string $snippet
   *   The snippet to unindent.
   *
   * @return string
   *   A string containing the unindented snippet.
   */
  private function unindent($snippet) {
    // Add a line break to include the first line for space/tab removal.
    $snippet = PHP_EOL . $snippet;
    while (!preg_match('/' . PHP_EOL . '\S/', $snippet)) {
      $snippet = preg_replace('/' . PHP_EOL . '[ \t]/', PHP_EOL, $snippet);
    }
    return trim($snippet);
  }

}
